@@ -1,5 +1,5 @@
 /* Main program of GNU linker.
-   Copyright (C) 1991, 92, 93, 94, 95, 96, 97, 98, 99, 2000
+   Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000
    Free Software Foundation, Inc.
    Written by Steve Chamberlain steve@cygnus.com
 
@@ -92,7 +92,7 @@ args_type command_line;
 ld_config_type config;
 
 static void remove_output PARAMS ((void));
-static boolean check_for_scripts_dir PARAMS ((char *dir));
+static boolean check_for_scripts_dir PARAMS ((char *dir, boolean append));
 static boolean add_archive_element PARAMS ((struct bfd_link_info *, bfd *,
 					    const char *));
 static boolean multiple_definition PARAMS ((struct bfd_link_info *,
@@ -206,6 +206,7 @@ main (argc, argv)
   command_line.rpath = NULL;
   command_line.warn_mismatch = true;
   command_line.check_section_addresses = true;
+  command_line.multilib_dir = NULL;
 
   /* We initialize DEMANGLING based on the environment variable
      COLLECT_NO_DEMANGLE.  The gcc collect2 program will demangle the
@@ -534,8 +535,9 @@ get_emulation (argc, argv)
    else return false.  */
 
 static boolean
-check_for_scripts_dir (dir)
+check_for_scripts_dir (dir, append)
      char *dir;
+     boolean append;
 {
   size_t dirlen;
   char *buf;
@@ -550,7 +552,7 @@ check_for_scripts_dir (dir)
   res = stat (buf, &s) == 0 && S_ISDIR (s.st_mode);
   free (buf);
   if (res)
-    ldfile_add_library_path (dir, false);
+    ldfile_add_library_path (dir, false, append);
   return res;
 }
 
@@ -568,7 +570,29 @@ set_scripts_dir ()
   char *end, *dir;
   size_t dirlen;
 
-  if (check_for_scripts_dir (SCRIPTDIR))
+  if (command_line.multilib_dir != NULL)
+    {
+      if (*command_line.multilib_dir == '/')
+	{
+	  dir = xmalloc (strlen (command_line.multilib_dir) + 1);
+	  strcpy (dir, command_line.multilib_dir);
+	}
+      else
+	{
+	  dir = xmalloc (strlen (SCRIPTDIR)
+			 + strlen (command_line.multilib_dir) + 2);
+	  strcpy (dir, SCRIPTDIR);
+	  strcat (dir, "/");
+	  strcat (dir, command_line.multilib_dir);
+	}
+
+      if (check_for_scripts_dir (dir, false))
+	return;
+      else
+	free (dir);
+    }
+
+  if (check_for_scripts_dir (SCRIPTDIR, true))
     /* We've been installed normally.  */
     return;
 
@@ -597,13 +621,13 @@ set_scripts_dir ()
   strncpy (dir, program_name, dirlen);
   dir[dirlen] = '\0';
 
-  if (check_for_scripts_dir (dir))
+  if (check_for_scripts_dir (dir, true))
     /* Don't free dir.  */
     return;
 
   /* Look for "ldscripts" in <the dir where our binary is>/../lib.  */
   strcpy (dir + dirlen, "/../lib");
-  if (check_for_scripts_dir (dir))
+  if (check_for_scripts_dir (dir, true))
     return;
 
   /* Well, we tried.  */
